from loguru import logger
from AbstractTsDbSinkHandler import TsDbSinkException, AbstractTsDbSinkHandler
import iso8601
import datetime


class EGM1mdTsDbSinkHandler(AbstractTsDbSinkHandler):
    def __init__(self, config, name, inQueue, experiment):
        super().__init__(config, name, inQueue, experiment)
        logger.info("constructor called")

    def sinkAction(self, dataObject):
        logger.info("database write operation start")
        try:
            # payload format: check dataformat.txt
            ts = iso8601.parse_date(dataObject.payload["ts"])
            dt = datetime.timedelta(microseconds=dataObject.payload["dt"])
            values = dataObject.payload["v"]
        
            stringBuffer = ''
            cnt = 0
            for rv in values:
                unsignedValue = rv & 0x00ffffff
                valid = (rv & 0x04000000) >> 26
                sign = (rv & 0x02000000) >> 25
                signedValue = unsignedValue if not sign else unsignedValue * -1
                stringBuffer += f"{ts}\t{self.experiment}\t{self.name}\t{rv}\t{valid}\t{signedValue}\n"
                ts += dt
                cnt += 1
            logger.info(f"{cnt} items")
            self.copy('egm1md', ('time', 'experiment', 'device', 'rawvalue', 'valid', 'value'), stringBuffer)
            logger.info("database write operation done")
        except iso8601.ParseError as e:
            logger.error(f"unable to parse timestamp, items dropped: {e}")
        except KeyError as e:
            logger.error(f"missing attribute in payload, items dropped: {e}")
        except TsDbSinkException as e:
            logger.error(f"error {e.code} when talking to database, items dropped: {e}")

