from loguru import logger
from AbstractTsDbSinkHandler import TsDbSinkException, AbstractTsDbSinkHandler
import iso8601


class FlowRigLogTsDbSinkHandler(AbstractTsDbSinkHandler):
    def __init__(self, config, name, inQueue, experiment):
        super().__init__(config, name, inQueue, experiment)
        logger.info("constructor called")

    def sinkAction(self, dataObject):
        logger.info("database write operation start")
        try:
            # payload format: check dataformat.txt
            ts = iso8601.parse_date(dataObject.payload["ts"])
            values = dataObject.payload["v"]

            stringBuffer = ''
            cnt = 0
            for valueItems in values:
                stringBuffer += f"{ts}\t{self.experiment}\t{self.name}\t{valueItems['n']}\t{valueItems['v']}\n"
                cnt += 1
            logger.info(f"{cnt} items")
            self.copy('flowriglog', ('time', 'experiment', 'device', 'name', 'value'), stringBuffer)
            logger.info("database write operation done")
        except iso8601.ParseError as e:
            logger.error(f"unable to parse timestamp, items dropped: {e}")
        except KeyError as e:
            logger.error(f"missing attribute in payload, items dropped: {e}")
        except TsDbSinkException as e:
            logger.error(f"error {e.code} when talking to database, items dropped: {e.msg}")


